#################################################################
# Script to generate macronutrient phenotype file for UKBB GWAS #
# Chloe Sarnowski, Chloe.Sarnowski@uth.tmc.edu                  #
# 08/02/21                                                      #
#################################################################

library('data.table')

# open UKBB main phenotype file
main=load('main_data.RData')

# open table with information on participants who withdrew consent
withdraw=read.table('withdrawal_files/w42614_20200204.csv')
ids=bd$f.eid

# removal of participants that have withdrawn
bd=bd[-which(ids%in%withdraw$V1),]
ids=bd$f.eid
sex=bd$f.31.0.0

#Total energy intake (conversion kJ in kcal)
energy=bd$f.100002.0.0/4.184
energy1=bd$f.100002.1.0/4.184
energy2=bd$f.100002.2.0/4.184
energy3=bd$f.100002.3.0/4.184
energy4=bd$f.100002.4.0/4.184

energy_all=data.frame(sex,energy,energy1,energy2,energy3,energy4)
tot_obs=rowSums(!is.na(energy_all[,-1]))
energy=bd$f.100002.0.0/4.184

# additional exclusions
ind_to_remove=c(which(energy<500 & sex=="Female"), which(energy>3500 & sex=="Female"), which(energy<800 & sex=="Male"), which(energy>4200 & sex=="Male"))
energy[ind_to_remove]=NA
energy1=bd$f.100002.1.0/4.184
ind_to_remove=c(which(energy1<500 & sex=="Female"), which(energy1>3500 & sex=="Female"), which(energy1<800 & sex=="Male"), which(energy1>4200 & sex=="Male"))
energy1[unique(ind_to_remove)]=NA
energy2=bd$f.100002.2.0/4.184
ind_to_remove=c(which(energy2<500 & sex=="Female"), which(energy2>3500 & sex=="Female"), which(energy2<800 & sex=="Male"), which(energy2>4200 & sex=="Male"))
energy2[unique(ind_to_remove)]=NA
energy3=bd$f.100002.3.0/4.184
ind_to_remove=c(which(energy3<500 & sex=="Female"), which(energy3>3500 & sex=="Female"), which(energy3<800 & sex=="Male"), which(energy3>4200 & sex=="Male"))
energy3[unique(ind_to_remove)]=NA
energy4=bd$f.100002.4.0/4.184
ind_to_remove=c(which(energy4<500 & sex=="Female"), which(energy4>3500 & sex=="Female"), which(energy4<800 & sex=="Male"), which(energy4>4200 & sex=="Male"))
energy4[unique(ind_to_remove)]=NA

energy_all=data.frame(sex,energy,energy1,energy2,energy3,energy4)
tot_obs_excl=rowSums(!is.na(energy_all[,-1]))
toto=tot_obs-tot_obs_excl
bd=bd[-which(toto>=2),]

energy_pheno=rowMeans(energy_all[,-1],na.rm=T)
energy_pheno=energy_pheno[-which(toto>=2)]

#Carb intake (conversion g in kcal)
carb=bd$f.100005.0.0*4
carb1=bd$f.100005.1.0*4
carb2=bd$f.100005.2.0*4
carb3=bd$f.100005.3.0*4
carb4=bd$f.100005.4.0*4

carb_all=data.frame(carb,carb1,carb2,carb3,carb4)
carb_pheno=rowMeans(carb_all,na.rm=T)

#Prot intake (conversion g in kcal)
prot=bd$f.100003.0.0*4
prot1=bd$f.100003.1.0*4
prot2=bd$f.100003.2.0*4
prot3=bd$f.100003.3.0*4
prot4=bd$f.100003.4.0*4

prot_all=data.frame(prot,prot1,prot2,prot3,prot4)
prot_pheno=rowMeans(prot_all,na.rm=T)

#Fat intake (conversion g in kcal)
fat=bd$f.100004.0.0*9
fat1=bd$f.100004.1.0*9
fat2=bd$f.100004.2.0*9
fat3=bd$f.100004.3.0*9
fat4=bd$f.100004.4.0*9

fat_all=data.frame(fat,fat1,fat2,fat3,fat4)
fat_pheno=rowMeans(fat_all,na.rm=T)

#covariates for analyses: age, sex, BMI and additional variables for sensitivity analyses
ethnicity=bd$f.21000.0.0
ids=bd$f.eid
sex=bd$f.31.0.0
age=bd$f.21022.0.0
bmi=bd$f.21001.0.0
age2=bd$f.21003.0.0
townsend=bd$f.189.0.0
smoke=bd$f.20116.0.0
alcohol=bd$f.1558.0.0
sleep=bd$f.1160.0.0
health=bd$f.2178.0.0

dataset=data.frame(ids,carb_pheno,fat_pheno,prot_pheno,sex,age,bmi,ethnicity,energy_pheno,age2,townsend,smoke,alcohol,sleep,health)

# open UKBB QC file (get PCs and genotyping array info)
qc <- fread("ukb_sqc_v2.txt")
# open one .fam file (here chr22)
ids=read.table('ukb42614_cal_chr22_v2_s488265.fam')
colnames(ids)[1]="ids"
qcok=data.frame(qc,ids[,1])
colnames(qcok)[69]="ids"
m=merge(dataset,qcok, by="ids")
ind_badqc=c(which(m$het.missing.outliers==1), which(m$putative.sex.chromosome.aneuploidy==1), which(m$Submitted.Gender!=m$Inferred.Gender))
# exclusion of participants not passing QC
dataset2=m[-ind_badqc,]

# selection of EUR ancestry participants based on self-reported ancestry
dataset2=dataset2[which(dataset2$ethnicity=="British" | dataset2$ethnicity=="White" | dataset2$ethnicity=="Irish" | dataset2$ethnicity=="Any other white background"),]

# exclusion of outliers at +/-6SD from the mean based on the first 4 PCs
outliers1=which(dataset2$PC1<(mean(dataset2$PC1)-6*sd(dataset2$PC1)) | dataset2$PC1>(mean(dataset2$PC1)+6*sd(dataset2$PC1)))
outliers2=which(dataset2$PC2<(mean(dataset2$PC2)-6*sd(dataset2$PC2)) | dataset2$PC2>(mean(dataset2$PC2)+6*sd(dataset2$PC2)))
outliers3=which(dataset2$PC3<(mean(dataset2$PC3)-6*sd(dataset2$PC3)) | dataset2$PC3>(mean(dataset2$PC3)+6*sd(dataset2$PC3)))
outliers4=which(dataset2$PC4<(mean(dataset2$PC4)-6*sd(dataset2$PC4)) | dataset2$PC4>(mean(dataset2$PC4)+6*sd(dataset2$PC4)))
outliers=c(outliers1,outliers2,outliers3,outliers4)
dataset2=dataset2[-unique(outliers),]

# exclusion of missing data
dataset3=dataset2[which(!is.na(dataset2$fat_pheno) & !is.na(dataset2$energy_pheno)),]

# generate macronutrient percentages
dataset3$carb_perc=dataset3$carb_pheno*100/(dataset3$energy_pheno)
dataset3$fat_perc=dataset3$fat_pheno*100/(dataset3$energy_pheno)
dataset3$prot_perc=dataset3$prot_pheno*100/(dataset3$energy_pheno)

# winsorizing at mean +/- 5SD

dataset3$carb_perc[which(dataset3$carb_perc<(mean(dataset3$carb_perc)-5*sd(dataset3$carb_perc)))]=mean(dataset3$carb_perc)-5*sd(dataset3$carb_perc)
dataset3$carb_perc[which(dataset3$carb_perc>(mean(dataset3$carb_perc)+5*sd(dataset3$carb_perc)))]=mean(dataset3$carb_perc)+5*sd(dataset3$carb_perc)

dataset3$fat_perc[which(dataset3$fat_perc<(mean(dataset3$fat_perc)-5*sd(dataset3$fat_perc)))]=mean(dataset3$fat_perc)-5*sd(dataset3$fat_perc)
dataset3$fat_perc[which(dataset3$fat_perc>(mean(dataset3$fat_perc)+5*sd(dataset3$fat_perc)))]=mean(dataset3$fat_perc)+5*sd(dataset3$fat_perc)

dataset3$prot_perc[which(dataset3$prot_perc<(mean(dataset3$prot_perc)-5*sd(dataset3$prot_perc)))]=mean(dataset3$prot_perc)-5*sd(dataset3$prot_perc)
dataset3$prot_perc[which(dataset3$prot_perc>(mean(dataset3$prot_perc)+5*sd(dataset3$prot_perc)))]=mean(dataset3$prot_perc)+5*sd(dataset3$prot_perc)

t=dataset3
t$health2=NA
t$health2[t$health=="Poor"]=1
t$health2[t$health=="Fair"]=2
t$health2[t$health=="Good"]=3
t$health2[t$health=="Excellent"]=4

t$former=NA
t$former[t$smoke=="Never"]=0
t$former[t$smoke=="Previous"]=1
t$former[t$smoke=="Current"]=0

t$smoker=NA
t$smoker[t$smoke=="Never"]=0
t$smoker[t$smoke=="Previous"]=0
t$smoker[t$smoke=="Current"]=1

t$alcohol2=NA
t$alcohol2[t$alcohol=="Never"]=0
t$alcohol2[t$alcohol=="Special occasions only"]=1
t$alcohol2[t$alcohol=="One to three times a month"]=2
t$alcohol2[t$alcohol=="Once or twice a week"]=3
t$alcohol2[t$alcohol=="Three or four times a week"]=4
t$alcohol2[t$alcohol=="Daily or almost daily"]=5

t$sleep[t$sleep<0]=NA

write.table(t,"Pheno_macronutrients_UKBB_for_GWAS.csv", quote=F,sep=",", row.names=F)
