####################################################################
# Example of command lines used to run association analyses        #
# with SAIGE in the UK Biobank with macronutrient intake phenotypes#
# Chloe Sarnowski, Chloe.Sarnowski@uth.tmc.edu                     #
# 08/02/2021                                                       #
####################################################################

#User has first to define:
#input dir (input_dir)
#location of SAIGE image (path_to_saige)
#location of UKBB PLINK files (ukbb_plink_dir)
#list of chromosomes to analyze (chroms)
#prefix of bgen files to use for each chr (bgen_prefixes), for instance ukb42614_imp_chr1_v3_s487284
#sample files for each chr (sample_files), for instance ukb42614_imp_chr1_v3_s487284.samplesaige

idx=$(( SGE_TASK_ID - 1 ))
chrom=${chroms[${idx}]}
bgen_prefix=${bgen_prefixes[${idx}]}
sample_file=${sample_files[${idx}]}


#1-NULL MODEL

#PROT
/opt/singularity/bin/singularity exec -B  /:/host_root -B "${TMPDIR}":/output -B "${input_dir}":/input ${saige_image} nullModel --phenoFile=/input/PROT_pheno.txt --phenoCol=prot_perc --sampleIDColinphenoFile=IID --plinkFile="${ukbb_plink_dir}":/ukb42614_imputed_autosomal_samples --outputPrefix=/output/PROT_null_model --covarColList=sex,age,bmi,townsend,smoker,former,alcohol2,sleep,health2,PC1,PC2,PC3,PC4,PC5,PC6,PC7,PC8,PC9,PC10,PC11,PC12,PC13,PC14,PC15,PC16,PC17,PC18,PC19,PC20,genotyping.array --traitType=quantitative

#CARB
/opt/singularity/bin/singularity exec -B  /:/host_root -B "${TMPDIR}":/output -B "${input_dir}":/input ${saige_image} nullModel --phenoFile=/input/CARB_pheno.txt --phenoCol=carb_perc --sampleIDColinphenoFile=IID --plinkFile="${ukbb_plink_dir}":/ukb42614_imputed_autosomal_samples --outputPrefix=/output/CARB_null_model --covarColList=sex,age,bmi,townsend,smoker,former,alcohol2,sleep,health2,PC1,PC2,PC3,PC4,PC5,PC6,PC7,PC8,PC9,PC10,PC11,PC12,PC13,PC14,PC15,PC16,PC17,PC18,PC19,PC20,genotyping.array --traitType=quantitative

#FAT
/opt/singularity/bin/singularity exec -B  /:/host_root -B "${TMPDIR}":/output -B "${input_dir}":/input ${saige_image} nullModel --phenoFile=/input/FAT_pheno.txt --phenoCol=fat_perc --sampleIDColinphenoFile=IID --plinkFile="${ukbb_plink_dir}":/ukb42614_imputed_autosomal_samples --outputPrefix=/output/FAT_null_model --covarColList=sex,age,bmi,townsend,smoker,former,alcohol2,sleep,health2,PC1,PC2,PC3,PC4,PC5,PC6,PC7,PC8,PC9,PC10,PC11,PC12,PC13,PC14,PC15,PC16,PC17,PC18,PC19,PC20,genotyping.array --traitType=quantitative


#2-ASSOCIATION ANALYSES

#PROT
/opt/singularity/bin/singularity exec -B  /:/host_root -B "${TMPDIR}":/output -B "${input_dir}":/input ${saige_image} association --bgenFile="${bgen_prefix}".bgen --bgenFileIndex="${bgen_prefix}".bgen.bgi --sampleFile="${sample_file}" --GMMATmodelFile=/output/PROT_null_model.rda --varianceRatioFile=/output/PROT_null_model.varianceRatio.txt --SAIGEOutputFile="/output/PROT_chr${chrom}_results" --chrom="${chrom}" --minMAC=3 --minMAF=0

#CARB
/opt/singularity/bin/singularity exec -B  /:/host_root -B "${TMPDIR}":/output -B "${input_dir}":/input ${saige_image} association --bgenFile="${bgen_prefix}".bgen --bgenFileIndex="${bgen_prefix}".bgen.bgi --sampleFile="${sample_file}" --GMMATmodelFile=/output/CARB_null_model.rda --varianceRatioFile=/output/CARB_null_model.varianceRatio.txt --SAIGEOutputFile="/output/CARB_chr${chrom}_results" --chrom="${chrom}" --minMAC=3 --minMAF=0

#FAT
/opt/singularity/bin/singularity exec -B  /:/host_root -B "${TMPDIR}":/output -B "${input_dir}":/input ${saige_image} association --bgenFile="${bgen_prefix}".bgen --bgenFileIndex="${bgen_prefix}".bgen.bgi --sampleFile="${sample_file}" --GMMATmodelFile=/output/FAT_null_model.rda --varianceRatioFile=/output/FAT_null_model.varianceRatio.txt --SAIGEOutputFile="/output/FAT_chr${chrom}_results" --chrom="${chrom}" --minMAC=3 --minMAF=0